@@ -1,24 +1,24 @@
 /***************************************************************************
-
+  
   c_x11.c
-
-  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
+  
+  (c) Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
-
+  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
-
+  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301, USA.
-
+  
 ***************************************************************************/
 
 #define __C_X11_C
@@ -68,11 +68,19 @@ static PROPERTY_FORMAT _property_format[] = {
 DECLARE_EVENT(EVENT_Change);
 DECLARE_EVENT(EVENT_Window);
 
-/*BEGIN_METHOD_VOID(CDESKTOP_init)
 
-	X11_init();
+/*static bool check_window(Window wid)
+{
+	if (wid == 0)
+	{
+		GB.Error("Null window");
+		return TRUE;
+	}
+	else
+		return FALSE;
+}*/
 
-END_METHOD*/
+#define GET_WINDOW(_arg) Window window = VARG(_arg); //if (check_window(window)) return
 
 BEGIN_METHOD(X11_FindWindow, GB_STRING title; GB_STRING klass; GB_STRING role)
 
@@ -175,12 +183,13 @@ BEGIN_METHOD(X11_GetWindowProperty, GB_INTEGER window; GB_STRING name)
 	GB_ARRAY array;
 	int i, count;
 	char *name;
-	Window window;
 	GB_VARIANT_VALUE ret;
 	
 	if (X11_init())
 		return;
 	
+	GET_WINDOW(window);
+	
 	prop = X11_intern_atom(GB.ToZeroString(ARG(name)), FALSE);
 	if (prop == None)
 	{
@@ -188,8 +197,6 @@ BEGIN_METHOD(X11_GetWindowProperty, GB_INTEGER window; GB_STRING name)
 		return;
 	}
 	
-	window = VARG(window);
-	
 	value = X11_get_property(window, prop, &type, &format, &count);
 	if (!value)
 	{
@@ -271,6 +278,7 @@ BEGIN_METHOD(X11_GetWindowProperty, GB_INTEGER window; GB_STRING name)
 
 END_METHOD
 
+
 BEGIN_METHOD(X11_SetWindowProperty, GB_INTEGER window; GB_STRING name; GB_STRING type; GB_VARIANT value)
 
 	Atom type;
@@ -279,7 +287,6 @@ BEGIN_METHOD(X11_SetWindowProperty, GB_INTEGER window; GB_STRING name; GB_STRING
 	int count;
 	void *data;
 	void *object;
-	Window window;
 	void *buffer = NULL;
 	#if OS_64BITS
 	long padded_value;
@@ -290,13 +297,13 @@ BEGIN_METHOD(X11_SetWindowProperty, GB_INTEGER window; GB_STRING name; GB_STRING
 	if (X11_init())
 		return;
 	
+	GET_WINDOW(window);
+	
 	prop = X11_intern_atom(GB.ToZeroString(ARG(name)), TRUE);
 	type = X11_intern_atom(GB.ToZeroString(ARG(type)), TRUE);
 	count = 0;
 	format = 0;
 	
-	window = VARG(window);
-	
 	switch(VARG(value).type)
 	{
 		case GB_T_STRING:
@@ -402,6 +409,7 @@ __ERROR:
 
 END_METHOD
 
+
 BEGIN_METHOD(X11_InternAtom, GB_STRING atom; GB_BOOLEAN create)
 
 	if (X11_init())
@@ -411,6 +419,7 @@ BEGIN_METHOD(X11_InternAtom, GB_STRING atom; GB_BOOLEAN create)
 
 END_METHOD
 
+
 BEGIN_METHOD(X11_GetAtomName, GB_INTEGER atom)
 
 	char *name;
@@ -424,6 +433,7 @@ BEGIN_METHOD(X11_GetAtomName, GB_INTEGER atom)
 
 END_METHOD
 
+
 BEGIN_METHOD(X11_SendClientMessage, GB_STRING message; GB_OBJECT data; GB_INTEGER window)
 
 	GB_ARRAY array;
@@ -481,6 +491,7 @@ BEGIN_METHOD(X11_SendClientMessage, GB_STRING message; GB_OBJECT data; GB_INTEGE
 
 END_METHOD
 
+
 BEGIN_PROPERTY(X11_EventFilter)
 
 	if (X11_init())
@@ -493,6 +504,7 @@ BEGIN_PROPERTY(X11_EventFilter)
 
 END_PROPERTY
 
+
 BEGIN_METHOD(X11_WatchWindow, GB_INTEGER window; GB_BOOLEAN watch)
 
 	XWindowAttributes attr;
@@ -501,15 +513,18 @@ BEGIN_METHOD(X11_WatchWindow, GB_INTEGER window; GB_BOOLEAN watch)
 	if (X11_init())
 		return;
 		
-	XGetWindowAttributes(X11_display, VARG(window), &attr);
+	GET_WINDOW(window);
+	
+	XGetWindowAttributes(X11_display, window, &attr);
 
 	if (VPROP(GB_BOOLEAN))
-		XSelectInput(X11_display, (Window)VARG(window), attr.your_event_mask | mask);
+		XSelectInput(X11_display, window, attr.your_event_mask | mask);
 	else
-		XSelectInput(X11_display, (Window)VARG(window), attr.your_event_mask & ~mask);
+		XSelectInput(X11_display, window, attr.your_event_mask & ~mask);
 
 END_METHOD
 
+
 BEGIN_METHOD(X11_GetWindowGeometry, GB_INTEGER window)
 
 	GB_ARRAY array;
@@ -518,15 +533,18 @@ BEGIN_METHOD(X11_GetWindowGeometry, GB_INTEGER window)
 	if (X11_init())
 		return;
 		
+	GET_WINDOW(window);
+
 	GB.Array.New(&array, GB_T_INTEGER, 4);
 	data = (int *)GB.Array.Get(array, 0);
 	
-	X11_get_window_geometry(VARG(window), &data[0], &data[1], &data[2], &data[3]);
+	X11_get_window_geometry(window, &data[0], &data[1], &data[2], &data[3]);
 	//XGetGeometry(X11_display, VARG(window), &root, &data[0], &data[1], (uint *)&data[2], (uint *)&data[3], &ignore, &ignore);
 	GB.ReturnObject(array);
 
 END_METHOD
 
+
 BEGIN_METHOD(X11_MakeIcon, GB_OBJECT data; GB_INTEGER width; GB_INTEGER height)
 
 	GB_ARRAY array;
@@ -593,22 +611,25 @@ BEGIN_METHOD(X11_MakeIcon, GB_OBJECT data; GB_INTEGER width; GB_INTEGER height)
 
 END_METHOD
 
+
 BEGIN_METHOD(X11_MinimizeWindow, GB_INTEGER window; GB_BOOLEAN minimized)
 
 	if (X11_init())
 		return;
 	
+	GET_WINDOW(window);
+	
 	if (VARG(minimized))
 	{
 		long state = IconicState;
 		
-		X11_send_client_message(X11_root, VARG(window),
+		X11_send_client_message(X11_root, window,
 			X11_intern_atom("WM_CHANGE_STATE", TRUE),
 			(char *)&state, 32, 1);
 	}
 	else
 	{
-		XMapWindow(X11_display, VARG(window));
+		XMapWindow(X11_display, window);
 	}
 
 END_METHOD
@@ -636,20 +657,31 @@ END_METHOD
 
 BEGIN_METHOD(X11_MoveWindow, GB_INTEGER window; GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_INTEGER h)
 
+	if (X11_init())
+		return;
+	
+	GET_WINDOW(window);
+	
 	if (MISSING(w) || MISSING(h))
-		XMoveWindow(X11_display, VARG(window), VARG(x), VARG(y));
+		XMoveWindow(X11_display, window, VARG(x), VARG(y));
 	else
-		XMoveResizeWindow(X11_display, VARG(window), VARG(x), VARG(y), VARG(w), VARG(h));
+		XMoveResizeWindow(X11_display, window, VARG(x), VARG(y), VARG(w), VARG(h));
 
 END_METHOD
 
+
 BEGIN_METHOD(X11_ResizeWindow, GB_INTEGER window; GB_INTEGER w; GB_INTEGER h)
 
-	XResizeWindow(X11_display, VARG(window), VARG(w), VARG(h));
+	if (X11_init())
+		return;
+	
+	GET_WINDOW(window);
+	XResizeWindow(X11_display, window, VARG(w), VARG(h));
 
 END_METHOD
 
 
+//-------------------------------------------------------------------------
 
 GB_DESC X11Desc[] =
 {
@@ -722,6 +754,7 @@ void WATCHER_event_filter(XEvent *e)
 	}
 }
 
+
 BEGIN_METHOD(X11Watcher_new, GB_INTEGER window; GB_STRING property)
 
 	if (X11_init())
@@ -737,6 +770,7 @@ BEGIN_METHOD(X11Watcher_new, GB_INTEGER window; GB_STRING property)
 
 END_METHOD
 
+
 BEGIN_METHOD_VOID(X11Watcher_free)
 
 	LIST_remove(&_watcher_list, WATCHER, &WATCHER->list);
@@ -745,6 +779,8 @@ BEGIN_METHOD_VOID(X11Watcher_free)
 
 END_METHOD
 
+//-------------------------------------------------------------------------
+
 GB_DESC X11WatcherDesc[] =
 {
   GB_DECLARE("X11Watcher", sizeof(CX11WATCHER)),
