@@ -1,24 +1,24 @@
 /***************************************************************************
+  
+  gbx_exec.c
+  
+  (c) Benoît Minisini <benoit.minisini@gambas-basic.org>
 
-	gbx_exec.c
-
-	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
-
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2, or (at your option)
-	any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software
-	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-	MA 02110-1301, USA.
-
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2, or (at your option)
+  any later version.
+  
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+  
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
+  
 ***************************************************************************/
 
 #define __GBX_EXEC_C
@@ -63,32 +63,14 @@ uint64_t EXEC_byref = 0;
 
 unsigned char EXEC_quit_value = 0; // interpreter return value
 
-#if 0
-bool EXEC_big_endian; // CPU endianness
-bool EXEC_debug_inside = FALSE; // debug inside components
-bool EXEC_debug_hold = FALSE; // hold execution at program end
-bool EXEC_task = FALSE; // I am a background task
-bool EXEC_profile = FALSE; // profiling mode
-bool EXEC_profile_instr = FALSE; // profiling mode at instruction level
-bool EXEC_trace = FALSE; // tracing mode
-bool EXEC_arch = FALSE; // executing an archive
-bool EXEC_fifo = FALSE; // debugging through a fifo
-bool EXEC_keep_library = FALSE; // do not unload libraries
-bool EXEC_main_hook_done = FALSE;
-bool EXEC_break_on_error = FALSE; // if we must break into the debugger as soon as there is an error.
-bool EXEC_in_event_loop = FALSE; // if we are in the event loop
-#if DO_NOT_CHECK_OVERFLOW
-#else
-bool EXEC_check_overflow = TRUE; // if we should check for overflow
-#endif
-#endif
-
 const char EXEC_should_borrow[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 2, 2, 0, 0, 1 };
 
 const char *EXEC_unknown_name;
 bool EXEC_unknown_property;
 char EXEC_unknown_nparam;
 
+//-------------------------------------------------------------------------
+
 void EXEC_init(void)
 {
 	union {
@@ -903,6 +885,32 @@ void EXEC_function_real()
 	EXEC_function_loop();
 }
 
+void EXEC_function_lambda(EXEC_LAMBDA *lambda)
+{
+	EXEC.class = lambda->class;
+	EXEC.object = lambda->object;
+	EXEC.index = lambda->index;
+	EXEC.func = &EXEC.class->load->func[lambda->index];
+	EXEC.nparam = 0;
+	
+	/*if (EXEC.func->fast && !JIT_exec(FALSE))
+		goto __RETURN;*/
+	
+	// We need to push a void frame, because EXEC_leave looks at *PC to know if a return value is expected
+	STACK_push_frame(&EXEC_current, 0);
+
+	PC = NULL;
+	GP = NULL;
+
+	ON_ERROR(error_EXEC_function_real)
+	{
+		EXEC_enter_quick();
+	}
+	END_ERROR
+
+	EXEC_function_loop();
+}
+
 void EXEC_function_loop()
 {
 	bool retry = FALSE;
@@ -933,7 +941,7 @@ void EXEC_function_loop()
 					PROPAGATE();
 				}
 
-				if (EXEC_break_on_error && EXEC_debug)
+				if (FLAG.break_on_error && FLAG.debug)
 					DEBUG.Main(TRUE);
 
 				if (ERROR->info.code == E_ASSERT)
@@ -1005,7 +1013,7 @@ void EXEC_function_loop()
 
 				ERROR_set_last(TRUE);
 
-				if (EXEC_debug && !FP->fast_linked && !STACK_has_error_handler())
+				if (FLAG.debug && !FP->fast_linked && !STACK_has_error_handler())
 				{
 					ERROR_hook();
 					
@@ -2058,7 +2066,7 @@ void EXEC_drop_vargs(void)
 
 void EXEC_set_got_error(bool err)
 {
-	EXEC_got_error = err;
+	FLAG.got_error = err;
 }
 
 PCODE *EXEC_on_goto_gosub(PCODE *pc)
@@ -2122,3 +2130,86 @@ void EXEC_gosub(PCODE *pc)
 	}
 }
 
+static void cb_later(EXEC_LAMBDA *lambda)
+{
+	*lambda->lock = 0; // Must unlock before calling the function, because the same AFTER DO can be called again.
+
+	if (!lambda->object || OBJECT_is_valid(lambda->object))
+		EXEC_function_lambda(lambda);
+	
+	OBJECT_UNREF(lambda->object);
+	IFREE(lambda);
+}
+
+static int cb_timer(EXEC_LAMBDA *lambda)
+{
+	int ret = FALSE;
+	bool stop_event = GAMBAS_StopEvent;
+	bool one_shot = ((CTIMER *)lambda->data)->one_shot;
+	
+	GAMBAS_StopEvent = FALSE;
+	
+	if (one_shot)
+		*lambda->lock = 0;
+	
+	EXEC_function_lambda(lambda);
+	
+	if (GAMBAS_StopEvent || one_shot)
+	{
+		if (!one_shot)
+			*lambda->lock = 0;
+		OBJECT_UNREF(lambda->object);
+		OBJECT_UNREF(lambda->data);
+		IFREE(lambda);
+		ret = TRUE;
+	}
+	
+	GAMBAS_StopEvent = stop_event;
+	return ret;
+}
+
+void EXEC_after_every(int op, int func, int var)
+{
+	EXEC_LAMBDA *lambda;
+	double delay;
+	char *lock;
+	
+	if (OP)
+		lock = &OP[CP->load->dyn[var].pos];
+	else
+		lock = (char *)CP->stat + CP->load->stat[var].pos;
+	
+	if (*lock)
+		return;
+	
+	*lock = 1;
+	
+	ALLOC(&lambda, sizeof(EXEC_LAMBDA));
+	
+	lambda->class = CP;
+	lambda->object = OP;
+	lambda->index = func;
+	lambda->lock = lock;
+	
+	OBJECT_REF_CHECK(OP);
+	
+	if (op != CAE_LATER)
+	{
+		delay = SUBR_get_float(SP - 1);
+		SP--;
+		if (delay <= 0)
+			op = CAE_LATER;
+	}
+	
+	if (op != CAE_LATER)
+	{
+		CTIMER *timer = CTIMER_every((int)(delay * 1000), (GB_TIMER_CALLBACK)cb_timer, (intptr_t)lambda);
+		timer->one_shot = op == CAE_AFTER;
+		lambda->data = timer;
+	}
+	else
+	{
+		EVENT_post(cb_later, (intptr_t)lambda);
+	}
+}
+
